classdef AntennaBuilder < handle
    %% AntennaBuilder AntennaBuilder main class
    %
    % Included in AToM, info@antennatoolbox.com
    %  2018, Vit Losenicky, CTU in Prague, vit.losenicky@antennatoolbox.com
    %  2018, Michal Masek, CTU in Prague, michal.masek@antennatoolbox.com
    
    properties (Access = private, Hidden = true)
        GUI
    end
    
    properties (GetAccess = {?GUI}, Hidden = true)
        isGUIOpened
        
        %% task settings
        Map
        FedPixels
        FeedingSettings
        FeedingArrows
        Mesh
        BasisFcns
        ka
        Operators
        Qlim = 21.7; %FIXME - load from somewhere
        
        %% selection
        PixelValues
        
        %% FOPS
        FOPS
        
        %% solutions
        Solutions
        
        %% solutions saving
        Filename
        Timer
        
        %% user
        User = struct('name', 'Default Name', 'email', '', ...
            'color', rand(1,3));
    end
    
    properties
        %% AntennaBuilder settings
        DisplayIDs = false
        Control = 'teamviewer'
        Closing = true;
        
        %% History window settings
        HistoryYFcn = @(y) y;
        HistoryYScale = 'log';
        HistoryYLim = @(Qlim, lastResults) min([0.8 1000] * Qlim, [Qlim, max(lastResults)]);
        HistoryNBars = 20;
        HistoryXTicksSpacing = 5;
    end
    
    methods
        function obj = AntennaBuilder()
            %% control presence of AToM
            if exist('AToM', 'dir') < 2
                disp('-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~');
                disp('AntennaBuilder: It is not possible to run AntennaBuilder.');
                disp(['You need to download ' ...
                    '<a href="http://www.antennatoolbox.com/order">AToM</a> first.']);
                disp('-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~');
                obj.delete;
                return
            end
            
            obj.User.name = obj.randomName();
            
            obj.isGUIOpened = 0;
            obj.Map = [];
            obj.FedPixels = [];
            obj.Mesh = [];
            obj.BasisFcns = [];
            obj.PixelValues = [];
            obj.ka = [];
            obj.Operators = [];
            
            %% solutions
            obj.Solutions = {};
            
            %% solutions saving
            obj.Filename = strrep(strrep(char(datetime), ' ', '_'), ...
                ':', '-');
            obj.Timer = timer();
            obj.Timer.StartFcn = @(a, b) obj.timerSave;
            obj.Timer.TimerFcn = @(a, b) '';
            
            
            obj.GUI = GUI(obj);
        end
        
        function exit(obj)
            obj.delete;
        end
    end
    
    methods (Hidden = true)
        function delete(obj)
            %% delete gui
            obj.GUI.delete;
            
            %% Clear AntennaBuilder instance from "base workspace"
            baseContent = evalin('base', 'whos');
            logInd = strcmp({baseContent.class}, 'AntennaBuilder');
            varName = {baseContent(logInd).name};
            if ~isempty(varName)
                evalin('base', ['clear(''',varName{1},''')']);
            end
        end
    end
    
    methods
        function set.DisplayIDs(obj, value)
            if value == 1
                obj.DisplayIDs = true;
                setStr = 'ON';
            else
                obj.DisplayIDs = false;
                setStr = 'OFF';
            end
            
            obj.update;
            fprintf('AntennaBuilder: Pixel IDs display %s \n', setStr);
        end
        
        function set.Control(obj, value)
            if any(strcmpi(value, {'mouse', 'm'}))
                obj.Control = 'mouse';
                setStr = 'MOUSE';
            elseif any(strcmpi(value, {'remote', 'teamviewer', 'r'}))
                obj.Control = 'teamviewer';
                setStr = 'TEAMVIEWER';
            elseif any(strcmpi(value, {'tablet', 't'}))
                obj.Control = 'tablet';
                setStr = 'TABLET';
            end
            
            obj.update;
            fprintf('AntennaBuilder: Control mode %s \n', setStr);
        end
        
        function set.Closing(obj, value)
            if value == 1
                obj.Closing = true;
                setStr = 'ON';
            else
                obj.Closing = false;
                setStr = 'OFF';
            end
            
            obj.update;
            fprintf('AntennaBuilder: Possibility to close %s \n', setStr);
        end
        
        function set.HistoryYScale(obj, value)
            if strcmpi(value, 'linear')
                obj.HistoryYScale = 'linear';
                setStr = 'LINEAR';
            else
                obj.HistoryYScale = 'log';
                setStr = 'LOG';
            end
            
            obj.update;
            fprintf('AntennaBuilder: History YScale set to %s \n', setStr);
        end
        
        function set.HistoryYFcn(obj, value)
            try
                if ischar(value)
                    value = str2func(value);
                end
                
                if nargin(value) == 1
                    obj.HistoryYFcn = value;
                end
                setStr = func2str(obj.HistoryYFcn);
            catch
                setStr = func2str(obj.HistoryYFcn);
                fprintf(['AntennaBuilder: It was not possible to set'...
                    'HistoryYFcn \n'], setStr);
            end
            
            obj.update;
            fprintf('AntennaBuilder: History HistoryYFcn set to %s \n', ...
                setStr);
        end
        
        function set.HistoryYLim(obj, value)
            try
                if ischar(value)
                    value = str2func(value);
                end
                
                if nargin(value) == 2
                    obj.HistoryYLim = value;
                end
                setStr = func2str(obj.HistoryYLim);
            catch
                setStr = func2str(obj.HistoryYLim);
                fprintf(['AntennaBuilder: It was not possible to set'...
                    'HistoryYLim \n'], setStr);
            end
            
            obj.update;
            fprintf('AntennaBuilder: History HistoryYLim set to %s \n', ...
                setStr);
        end
        
        function set.HistoryNBars(obj, value)
            if isnumeric(value) && mod(value, 1) == 0 && value > 1 && ...
                    length(value) == 1
                setStr = num2str(value);
                obj.HistoryNBars = value;
            else
                setStr = num2str(obj.HistoryNBars);
            end
            
            obj.update;
            fprintf('AntennaBuilder: History HistoryNBars set to %s \n',...
                setStr);
        end
        
        function set.HistoryXTicksSpacing(obj, value)
            if isnumeric(value) && value > 0 && length(value) == 1
                setStr = num2str(value);
                obj.HistoryXTicksSpacing = value;
            else
                setStr = num2str(obj.HistoryXTicksSpacing);
            end
            obj.update;
            fprintf(['AntennaBuilder: History HistoryXTicksSpacing '...
                'set to %s \n'], setStr);
        end
        
        
    end
    
    methods
        reset(obj);
        update(obj);
        open(obj);
        
        setTask(obj, map, fedPixels, mesh, basisFcns, ka, operators);
        addFeeder(obj, fedPixels, amplitude, phase);
        removeFeeder(obj, feeder);
        
        [feedingPosition, V] = feedingPosition(obj);
        
        solve(obj);
        runGA(obj);
        runSymGA(obj);
        saveSolution(obj, solution, duplicityEnabled);
        timerSave(obj);
        
        setUser(obj, name, email, color);
        
        function loadSolution(obj, solution)
            obj.GUI.loadSolution(solution);
        end
    end
    
    methods(Static)
        randName = randomName();
    end
end

