function runSymGA(obj)
%% runGA runs FOPS NSGA II
%
% Included in AToM, info@antennatoolbox.com
%  2018, Miloslav Capek, CTU in Prague, miloslav.capek@antennatoolbox.com
%  2018, Vit Losenicky, CTU in Prague, vit.losenicky@antennatoolbox.com

%% control presence of fops
if exist('fops', 'dir') < 2
    disp('-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~');
    disp('AntennaBuilder: It is not possible to run genetic algorithm.');
    disp(['You need to download ' ...
        '<a href="http://www.antennatoolbox.com/order">FOPS</a> first.']);
    disp('-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~');
    return
end

%% Optimization data
nAgents = 250; % number of agents for multi-criteria optimization
nIters  = 100; % number of iterations for multi-criteria optimization

Map = obj.Map;
optimizedPixels = setdiff(1:length(Map), obj.FedPixels);
optimizedPixelsTemp = setdiff(1:length(Map), obj.FedPixels);

%% symmetry treatment =====================================================
optimizedPixels(optimizedPixels > 64) = [];

temp = reshape(1:64, 8, []);
symMat = [temp fliplr(temp)];
%==========================================================================

%%
optimizedMap = pixeling.getMap( ...
    cellfun(@(x) x.pPixel, Map(optimizedPixelsTemp), ...
    'UniformOutput', false), obj.Mesh, obj.BasisFcns, 3);

pDOF  = size(optimizedMap, 1)/2;

[~, V] = obj.feedingPosition;

optData.st.Mesh          = obj.Mesh;
optData.st.BF            = obj.BasisFcns;
optData.op.ka            = obj.ka;
optData.op.Z             = obj.Operators.Z;
optData.op.omWm          = obj.Operators.omXm;
optData.op.omWe          = obj.Operators.omXe;
optData.op.V             = V;
optData.Map              = optimizedMap;
optData.optimizedPixels  = optimizedPixels;
optData.symmetryMat      = symMat;
optData.options.minArea  = 4;
optData.options.pDOF     = pDOF;
optData.options.totArea  = ...
    sum(pixeling.getMapData(optimizedMap, 'pArea', [], true));
optData.t0               = clock;
optData.history.removedPixels = zeros(size(optimizedMap, 1), 1);
optData.fitnessList = {@(Zr, Zir, omXmr, omXer, Vr) ...
    EMiT_fitnessFunction_lowlevel(Zr, Zir, omXmr, omXer, Vr)};

%% Optimization task (NEW FOPS):
fops = FOPS(false);
fops.showProgress = false;
fops.activeHistory = true;
tasksToBeSolved = @(x, fops) {EMiT_fitnessFunction(x, fops, obj)};
problemStruct = struct(...
    'limits', [zeros(1, pDOF); ones(1, pDOF)], ...
    'fitness', tasksToBeSolved, ...
    'fullControl', true, ...
    'isVectorized', true, ...
    'name', 'EMiT_CEM_challenge');
settings.nIters  = nIters;
settings.nagents = nAgents;
settings.BP      = 1;
fops.addTask({problemStruct}, {'NSGAII'}, settings, 'fminfcn1', optData);

%% Run the optimization
fops.runTask;
end

function minQ = EMiT_fitnessFunction(genes, Task, obj)
%%
%
% (M.C., 2017)

% Copy all required data from the userData structure:
Map              = Task.userData{1}.Map;
optimizedPixels  = Task.userData{1}.optimizedPixels;
symMat           = Task.userData{1}.symmetryMat;

% Symm treatement =========================================================
symVec = reshape(symMat, 1, []);
symVec(symVec > 63) = [];
genesSymetrized = genes(:, symVec);

optimizedPixels2 = optimizedPixels;
temp = flipud(reshape(1:128, 8, []));
for thisCol = 1:8
    for thisRow = 1:8
        optimizedPixels2(optimizedPixels2 == temp(thisRow, thisCol)) = ...
            temp(thisRow, end + 1 - thisCol);
    end
end
optimizedPixels2 = [optimizedPixels2 NaN];
optimizedPixels2 = fliplr(reshape(flipud(reshape(optimizedPixels2, 8, [])), 1, []));
optimizedPixels2(isnan(optimizedPixels2)) = [];
optimizedPixels = [optimizedPixels optimizedPixels2];
% =========================================================================


% --- FITNESS FCN DEPENDENT: ----------------------------------------------
Z       = Task.userData{1}.op.Z;
omWm    = Task.userData{1}.op.omWm;
omWe    = Task.userData{1}.op.omWe;
V       = Task.userData{1}.op.V;
% --- END fitness fcn dependent -------------------------------------------
options       = Task.userData{1}.options;
fitness       = Task.userData{1}.fitnessList{1,1};

% this iteration:
CutPixels       = logical(genesSymetrized(:, 1:2*options.pDOF));
SmallPixels     = false(size(CutPixels));
Edges        = cell(size(CutPixels, 1), 1);

% Allocations:
nAgents = size(CutPixels, 1);
Qm      = inf(nAgents, 1);
Qe      = inf(nAgents, 1);
I       = cell(nAgents, 1);
% PARFOR for all agents in this iteration:
parfor a = 1:nAgents
    % FLOODfilling (comment following line for disabling!)
    SmallPixels(a, :) = pixeling.pixelsWithAreaSmallerThan( ...
        Map, find(~CutPixels(a, :)), options.minArea);
    
    CutPixels(a, :) = CutPixels(a, :) | SmallPixels(a, :);
    
    % Use Map to map between pixels and edges:
    CutEdges = pixeling.edgesInPixels(Map, CutPixels(a, :));
    Edges{a} = setdiff(1:length(Z), CutEdges);
    
    % Pixelling:
    [Zr, omXmr, omXer, Vr] = pixeling.removeEdges(CutEdges, Z, omWm, ...
        omWe, V);
    
    % Optimization
    if size(Zr > 0, 1)
        [Qm(a), Qe(a), I{a}] = fitness(Zr, inv(Zr), omXmr, omXer, Vr);
    end
end

minQ = 2 * max(Qm, Qe);

% Statistics:
Task.userData{1}.history.removedPixels = ...
    Task.userData{1}.history.removedPixels + sum(CutPixels, 1).';

% =========================================================================
% Show counter and print contemporary results:
thisIter      = Task.currentIter;
elapsedTime   = etime(clock, Task.userData{1}.t0);
remainingTime = (Task.nIters - thisIter) / thisIter * elapsedTime;

bestAgent = find(minQ == min(minQ), 1);

% DISPLAY OPT progress:
fprintf(1, ['IT: %d | min. Q: %f (ag. %d),' ...
    ' elapsed time: %fs, remaining time: %fs \n'], thisIter, ...
    min(minQ), bestAgent, elapsedTime, remainingTime);

obj.PixelValues(optimizedPixels(CutPixels(bestAgent,:))) = 0;
obj.PixelValues(optimizedPixels(~CutPixels(bestAgent,:))) = 1;
obj.PixelValues(optimizedPixels(SmallPixels(bestAgent,:))) = 2;

iVec = zeros(length(Z), 1);
iVec(Edges{bestAgent}) = I{bestAgent};
solution.iVec = iVec;
solution.Qm = Qm(bestAgent);
solution.Qe = Qe(bestAgent);
solution.Qmax = minQ(bestAgent);
solution.author = 'FOPS-NSGAII';

obj.saveSolution(solution, 1);
obj.GUI.show;
drawnow;

end

function [Qm, Qe, I] = EMiT_fitnessFunction_lowlevel(Z, Zinv, omXm, omXe, V)
%% Low-level fitness function calculating tuned Q, Xin/Rin and resistance
% (M.C., 2017)

I = Zinv * V;

Prad = real(I' * real(Z) * I) / 2;
omWm = real(I' * omXm * I) / 4;
omWe = real(I' * omXe * I) / 4;
Qm   = omWm / Prad;
Qe   = omWe / Prad;
end

